#!@PYTHON@

#
# SPDX-License-Identifier: BSD-3-Clause
# Copyright Contributors to the OpenEXR Project.
#

import os
import sys
from pathlib import Path

from imath import *

import imathnumpy
import numpy as np

testList = []

def numpyType(ia):
    podTypeDict = {
        SignedCharArray :    np.int8,
        UnsignedCharArray :  np.uint8,
        ShortArray :         np.int16,
        UnsignedShortArray : np.uint16,
        IntArray :           np.int32,
        UnsignedIntArray :   np.uint32,
        FloatArray :         np.float32,
        DoubleArray :        np.float64,
        V2sArray :           np.int16,
        V2iArray :           np.int32,
        V2fArray :           np.float32,
        V2dArray :           np.float64,
        V3sArray :           np.int16,
        V3iArray :           np.int32,
        V3fArray :           np.float32,
        V3dArray :           np.float64,
        V4sArray :           np.int16,
        V4iArray :           np.int32,
        V4fArray :           np.float32,
        V4dArray :           np.float64,
        C3cArray :           np.uint8,
        C3fArray :           np.float32,
        C4cArray :           np.uint8,
        C4fArray :           np.float32,
        IntArray2D :         np.int32,
        FloatArray2D :       np.float32,
        DoubleArray2D :      np.float64,
        Color4cArray2D :     np.uint8,
        Color4fArray2D :     np.float32,
    }

    return podTypeDict[type(ia)]

def baseTypeSize(ia):
    baseTypeSizeDict = {
        SignedCharArray :    1,
        UnsignedCharArray :  1,
        ShortArray :         1,
        UnsignedShortArray : 1,
        IntArray :           1,
        UnsignedIntArray :   1,
        FloatArray :         1,
        DoubleArray :        1,
        V2sArray :           2,
        V2iArray :           2,
        V2fArray :           2,
        V2dArray :           2,
        V3sArray :           3,
        V3iArray :           3,
        V3fArray :           3,
        V3dArray :           3,
        V4sArray :           4,
        V4iArray :           4,
        V4fArray :           4,
        V4dArray :           4,
        C3cArray :           3,
        C3fArray :           3,
        C4cArray :           4,
        C4fArray :           4,
        Color4cArray2D :     4,
        Color4fArray2D :     4
    }

    return baseTypeSizeDict[type(ia)]


###############################################################################

def testScalarArrayToNumpy():
    '''
    Tests the wrapping of PyImath scalar array types to numpy arrays.
    '''
    aDim = 20

    # Test all supported array types
    arrayTestTypes = [
        SignedCharArray,
        UnsignedCharArray,
        ShortArray,
        UnsignedShortArray,
        IntArray,
        UnsignedIntArray,
        FloatArray,
        DoubleArray]

    for arrayType in arrayTestTypes:
        print("\nTesting: %s conversion to numpy" % arrayType.__name__)

        ia = arrayType(aDim)
        refType  = numpyType(ia)
        refShape = (aDim,)
        
        for i in range(aDim):
            ia[i] = i
        na = imathnumpy.arrayToNumpy(ia)
        
        print("%s == %s" % (str(na.dtype), str(refType)))
        assert(na.dtype == refType)

        print("%s == %s" % (str(na.shape), str(refShape)))
        assert(na.shape == refShape)

        for i in range(aDim):
            assert(na[i] == ia[i])
        print(na)

testList.append(testScalarArrayToNumpy)

###############################################################################

def testVectorArrayToNumpy():
    '''
    Tests the wrapping of PyImath vector array types to numpy arrays.
    '''
    aDim = 20

    # Test all supported array types
    arrayTestTypes = [
        V2sArray,
        V2iArray,
        V2fArray,
        V2dArray,
        V3sArray,
        V3iArray,
        V3fArray,
        V3dArray,
        V4sArray,
        V4iArray,
        V4fArray,
        V4dArray,
        C3cArray,
        C3fArray,
        C4cArray,
        C4fArray]

    for arrayType in arrayTestTypes:
        print("\nTesting: %s conversion to numpy" % arrayType.__name__)

        ia = arrayType(aDim)
        refType         = numpyType(ia)
        refBaseTypeSize = baseTypeSize(ia)
        refShape        = (aDim, refBaseTypeSize)

        for i in range(aDim):
            for j in range(refBaseTypeSize):
                ia[i][j] = i + 2 ** j
        na = imathnumpy.arrayToNumpy(ia)
        
        print("%s == %s" % (str(na.dtype), str(refType)))
        assert(na.dtype == refType)

        print("%s == %s" % (str(na.shape), str(refShape)))
        assert(na.shape == refShape)

        for i in range(aDim):
            for j in range(refBaseTypeSize):
                assert(na[i][j] == ia[i][j])
        print(na)

testList.append(testVectorArrayToNumpy)

###############################################################################

def testScalarArray2DToNumpy():
    '''
    Tests the wrapping of PyImath scalar2D array types to numpy arrays.
    '''
    aDim1 = 10
    aDim2 = 5

    # Test all supported array types
    arrayTestTypes = [
        IntArray2D,
        FloatArray2D,
        DoubleArray2D]

    for arrayType in arrayTestTypes:
        print("\nTesting: %s conversion to numpy" % arrayType.__name__)

        ia = arrayType(aDim1, aDim2)
        refType  = numpyType(ia)
        refShape = (aDim2, aDim1)

        for y in range(aDim2):
            for x in range(aDim1):
                ia[(x, y)] = x + 2 * y
        na = imathnumpy.arrayToNumpy(ia)

        print("%s == %s" % (str(na.dtype), str(refType)))
        assert(na.dtype == refType)

        print("%s == %s" % (str(na.shape), str(refShape)))
        assert(na.shape == refShape)

        for y in range(aDim2):
            for x in range(aDim1):
                assert(na[y][x] == ia.item(x, y))
        print(na)

testList.append(testScalarArray2DToNumpy)

###############################################################################

def testVectorArray2DToNumpy():
    '''
    Tests the wrapping of PyImath vector2D array types to numpy arrays.
    '''
    aDim1 = 10
    aDim2 = 5

    # Test all supported array types
    arrayTestTypes = [
        Color4cArray2D,
        Color4fArray2D]

    for arrayType in arrayTestTypes:
        print("\nTesting: %s conversion to numpy" % arrayType.__name__)

        ia = arrayType(aDim1, aDim2)
        refType         = numpyType(ia)
        refBaseTypeSize = baseTypeSize(ia)
        refShape        = (aDim2, aDim1, refBaseTypeSize)

        for y in range(aDim2):
            for x in range(aDim1):
                for k in range(refBaseTypeSize):
                    ia.item(x, y)[k] = x + 2 * y + k
        na = imathnumpy.arrayToNumpy(ia)

        print("%s == %s" % (str(na.dtype), str(refType)))
        assert(na.dtype == refType)

        print("%s == %s" % (str(na.shape), str(refShape)))
        assert(na.shape == refShape)

        for y in range(aDim2):
            for x in range(aDim1):
                for k in range(refBaseTypeSize):
                    assert(na[y][x][k] == ia.item(x, y)[k])
        print(na)

testList.append(testVectorArray2DToNumpy)

###############################################################################

def testBufferProtocol():
    '''
    Numpy arrays and FixedArrays both now support the Python buffer protocol.
    This test will exercise the FixedArray implementation of the protocol
    against numpy arrays since only very limited testing of the protocol is
    possible in PyImathTest .
    '''
    def verify (x, X, factor=1., shared=True):

        assert (len(x) == len(X))
        for i in range(10):
            #  Make sure each element is the proper factor of the index, this
            # will help make sure all the indexing and striding is correct.
            assert (X[i] == i*factor)
            assert (X[i] == x[i])
        
        #  Buffers can be shared, so modifying the imath array should also
        # be operating on the numpy array...
        for i in range(10):
            x[i] *= 10
        for i in range(10):
            assert ((shared and (X[i] == x[i])) or (not shared and (X[i]*10 == x[i])))


    def verifyVector (v, V, cloned=False):

        assert (len(v) == len(V))
        for i in range(10):
            assert (len(v[i]) == len(V[i]))
            for j in range(len(v[i])): 
                assert (V[i][j] == v[i][j])
        
        #  Buffers should be shared, so modifying the imath array should also
        # be operating on the numpy array...
        for i in range(10):
            v[i] *= 10
        for i in range(10):
            for j in range(len(v[i])): 
                assert ((not cloned and (V[i][j] == v[i][j])) or (cloned and (V[i][j]*10 == v[i][j])))

        #  Verify that imath striding is properly mapped to numpy...
        # Note that we modified the elements by a factor of 10 above, so when
        # we check each element against the index we need to account for this.
        Vx = np.asarray (v.x)
        Vy = np.asarray (v.y)
        if cloned:
            # Factors will have already been applied...
            factor = 1000.
        else:
            factor = 10.
        verify (v.x, Vx, factor, shared=True)
        verify (v.y, Vy, factor*10., shared=True)

        if (len(v[0]) == 3):
            Vz = np.asarray (v.z)
            verify (v.z, Vz, factor*10.*10., shared=True)

    # .........................................................................

    a = UnsignedCharArray (10)
    for i in range(10):
        a[i] = i
    aNP = np.asarray (a)
    verify (a, aNP)

    a = IntArray (10)
    for i in range(10):
        a[i] = i
    aNP = np.asarray (a)
    verify (a, aNP)

    #  Use the BufferProtocol to construct a deepcopy of the numpy
    # array as a FixedArray.
    aNPclone = IntArrayFromBuffer (aNP)
    verify (aNPclone, aNP, factor=10., shared=False)

    a = FloatArray (10)
    for i in range(10):
        a[i] = float(i)
    aNP = np.asarray (a)
    verify (a, aNP)
    
    #  Use the BufferProtocol to construct a deepcopy of the numpy
    # array as a FixedArray.
    aNPclone = FloatArrayFromBuffer (aNP)
    verify (aNPclone, aNP, factor=10., shared=False)

    a = DoubleArray (10)
    for i in range(10):
        a[i] = float(i)
    aNP = np.asarray (a)
    verify (a, aNP)
    
    #  Use the BufferProtocol to construct a deepcopy of the numpy
    # array as a FixedArray.
    aNPclone = DoubleArrayFromBuffer (aNP)
    verify (aNPclone, aNP, factor=10., shared=False)

    v = V2iArray (10)
    for i in range(10):
        v[i] = V2i (i,i*10)
    vNP = np.asarray (v)
    verifyVector (v, vNP)

    #  Use the BufferProtocol to construct a deepcopy of the numpy
    # array as a FixedArray.
    vNPclone = V2iArrayFromBuffer (vNP)
    verifyVector (vNPclone, vNP, cloned=True)

    v = V2fArray (10)
    for i in range(10):
        v[i] = V2f (i,i*10)
    vNP = np.asarray (v)
    verifyVector (v, vNP)

    #  Use the BufferProtocol to construct a deepcopy of the numpy
    # array as a FixedArray.
    vNPclone = V2fArrayFromBuffer (vNP)
    verifyVector (vNPclone, vNP, cloned=True)

    v = V2dArray (10)
    for i in range(10):
        v[i] = V2d (i,i*10)
    vNP = np.asarray (v)
    verifyVector (v, vNP)

    #  Use the BufferProtocol to construct a deepcopy of the numpy
    # array as a FixedArray.
    vNPclone = V2dArrayFromBuffer (vNP)
    verifyVector (vNPclone, vNP, cloned=True)

    v = V3iArray (10)
    for i in range(10):
        v[i] = V3i (i,i*10,i*100)
    vNP = np.asarray (v)
    verifyVector (v, vNP)

    #  Use the BufferProtocol to construct a deepcopy of the numpy
    # array as a FixedArray.
    vNPclone = V3iArrayFromBuffer (vNP)
    verifyVector (vNPclone, vNP, cloned=True)

    v = V3fArray (10)
    for i in range(10):
        v[i] = V3f (i,i*10,i*100)
    vNP = np.asarray (v)
    verifyVector (v, vNP)

    #  Use the BufferProtocol to construct a deepcopy of the numpy
    # array as a FixedArray.
    vNPclone = V3fArrayFromBuffer (vNP)
    verifyVector (vNPclone, vNP, cloned=True)

    v = V3dArray (10)
    for i in range(10):
        v[i] = V3d (i,i*10,i*100)
    vNP = np.asarray (v)
    verifyVector (v, vNP)

    #  Use the BufferProtocol to construct a deepcopy of the numpy
    # array as a FixedArray.
    vNPclone = V3dArrayFromBuffer (vNP)
    verifyVector (vNPclone, vNP, cloned=True)

testList.append (testBufferProtocol)

# -------------------------------------------------------------------------
# Main loop

for test in testList:
    print("")
    print("Running %s" % test.__name__)
    test()

print("done.")

# Local Variables:
# mode:python
# End:



#!/usr/bin/env python3

import os, site, sys, glob


def test_import():

    try:
        import imathnumpy
        print()
        print(f"imathnumpy imported from {imathnumpy.__file__}")
        print("ok")
    except Exception as e:
        print(f"{__file__} failed. Can't import imathnumpy: {e}")
        sys.exit(1)

test_import()



# SPDX-License-Identifier: BSD-3-Clause
# Copyright Contributors to the OpenEXR Project.

import sys, os, glob, unittest


cwd = os.getcwd()

print(sys.argv)

# Make sure we're in the correct directory to get the tests
main_dir = os.path.dirname(os.path.abspath(sys.argv[0]))
os.chdir( main_dir )

# For now, add the path we import from
# for imath paths and make sure they come
# before any other paths
sys.path.insert(1, os.path.abspath(sys.argv[1]))

testFiles = sys.argv[2:]

if not testFiles:
    testFiles = glob.glob('test*.py')

# Load all the tests
suite = unittest.TestSuite()

for file in testFiles:
    name = os.path.splitext(file)[0]
    __import__(name)
    test = unittest.defaultTestLoader.loadTestsFromName(name)
    suite.addTest(test)


# lets set it back before running so our Alembic files get dumped into
# our original working directory, instead of the source of our tests
os.chdir( cwd )

# Run the tests
runner = unittest.TextTestRunner(verbosity=2)
result = runner.run(suite)

if not result.wasSuccessful():
    exit(1)



# SPDX-License-Identifier: BSD-3-Clause
# Copyright Contributors to the OpenEXR Project.

# Move this back to test_m44.py once this has been evaluated
# for pybind11 AND boost python bindings

import unittest
from imath import V3f, M44f, Eulerf

class M44fTestCase(unittest.TestCase):
    def test_baseTypeEpsilon(self):
        result = M44f.baseTypeEpsilon()
        self.assertEqual(result, 1.1920928955078125e-07)

    def test_baseTypeLowest(self):
        result = M44f.baseTypeLowest()
        self.assertEqual(result, -3.4028234663852886e+38)

    def test_baseTypeMax(self):
        result = M44f.baseTypeMax()
        self.assertEqual(result, 3.4028234663852886e+38)

    def test_baseTypeSmallest(self):
        result = M44f.baseTypeSmallest()
        self.assertEqual(result, 1.1754943508222875e-38)

    def test_determinant(self):
        matrix = M44f(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16)
        result = matrix.determinant()
        expected_result = 0.0
        self.assertEqual(result, expected_result)

    def test_equalWithAbsError(self):
        matrix1 = M44f(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16)
        matrix2 = M44f(1.01, 2.02, 3.01, 4.02, 5.01, 6.02, 7.01, 8.02, 9.01, 10.02, 11.01, 12.02, 13.01, 14.02, 15.01, 16.02)
        result = M44f.equalWithAbsError(matrix1, matrix2, 0.05)
        self.assertTrue(result)

    def test_equalWithRelError(self):
        matrix1 = M44f(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16)
        matrix2 = M44f(1.01, 2.02, 3.01, 4.02, 5.01, 6.02, 7.01, 8.02, 9.01, 10.02, 11.01, 12.02, 13.01, 14.02, 15.01, 16.02)
        result = M44f.equalWithRelError(matrix1, matrix2, 0.02)
        self.assertTrue(result)

    def test_extractAndRemoveScalingAndShear(self):
        """
        check scale, shear and if the they were removed from the matrix
        """
        matrix = M44f()
        scaling, shear = V3f(), V3f()
        matrix.scale(V3f(1, 2, 3))
        M44f.extractAndRemoveScalingAndShear(matrix, scaling, shear)
        expected_scaling = V3f(1, 2, 3)
        self.assertEqual(scaling, expected_scaling)
        matrix = M44f()
        scaling, shear = V3f(), V3f()
        matrix.shear(V3f(1, 2, 3))
        M44f.extractAndRemoveScalingAndShear(matrix, scaling, shear)
        expected_shear = V3f(1, 2, 3)
        self.assertEqual(shear, expected_shear)
        self.assertEqual(matrix, M44f())

    def test_extractEulerXYZ(self):
        """
        TODO: doesn't work
        """
        matrix = M44f().rotate(Eulerf(45, 30, 15))
        # result = M44f.extractEulerXYZ(matrix) # ?
        result = Eulerf()
        M44f.extractEulerXYZ(matrix, result)
        expected_result = Eulerf(45, 30, 15)
        # self.assertEqual(result, expected_result)

    # def test_extractEulerZYX(self):
    #     matrix = M44f().rotate(Eulerf(45, 30, 15))
    #     result = M44f.extractEulerZYX(matrix)
    #     expected_result = Eulerf(15, 30, 45)
    #     self.assertEqual(result, expected_result)

    def test_extractSHRT(self):
        """
        TODO: do 4 tests?
        """
        matrix = M44f().setTranslation(V3f(1, 2, 3)).rotate(Eulerf(45, 30, 15)).setScale(V3f(2, 3, 4)).setShear(V3f(0.1, 0.2, 0.3))
        s, h, r, t = V3f(), V3f(), V3f(), V3f()
        M44f.extractSHRT(matrix, s, h, r, t)
        expected_scale = V3f(2, 3, 4)
        expected_shear = V3f(0.1, 0.2, 0.3)
        expected_rotation = Eulerf(45, 30, 15)
        expected_translation = V3f(1, 2, 3)
        # self.assertEqual(s, expected_scale)
        # self.assertEqual(h, expected_shear)
        # self.assertEqual(r, expected_rotation)
        # self.assertEqual(t, expected_translation)

    def test_extractScaling(self):
        matrix = M44f().setScale(V3f(2, 3, 4))
        result = V3f()
        M44f.extractScaling(matrix, result)
        expected_result = V3f(2, 3, 4)
        self.assertEqual(result, expected_result)

    def test_extractScalingAndShear(self):
        """
        same as test_extractAndRemoveScalingAndShear
        except the matrix doesn't change
        """
        matrix = M44f()
        scaling, shear = V3f(), V3f()
        matrix.scale(V3f(1, 2, 3))
        M44f.extractScalingAndShear(matrix, scaling, shear)
        expected_scaling = V3f(1, 2, 3)
        self.assertEqual(scaling, expected_scaling)
        matrix = M44f()
        scaling, shear = V3f(), V3f()
        matrix.shear(V3f(1, 2, 3))
        M44f.extractScalingAndShear(matrix, scaling, shear)
        expected_shear = V3f(1, 2, 3)
        self.assertEqual(shear, expected_shear)

    # def test_fastMinor(self):
    #     matrix = M44f(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16)
    #     result = M44f.fastMinor(matrix, 1, 2, 3, 1)
    #     expected_result = 0.0
    #     self.assertEqual(result, expected_result)

    # def test_gjInverse(self):
    #     matrix = M44f(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16)
    #     result = M44f.gjInverse(matrix)
    #     expected_result = M44f(-2.0, 1.0, 0.0, 0.0, 1.5, -1.0, 0.5, 0.0, -1.0, 1.0, 0.0, 0.0, 0.5, -0.5, 0.0, 0.0)
    #     self.assertEqual(result, expected_result)

    # def test_gjInvert(self):
    #     matrix = M44f(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16)
    #     result = matrix.gjInvert()
    #     expected_result = M44f(-2.0, 1.0, 0.0, 0.0, 1.5, -1.0, 0.5, 0.0, -1.0, 1.0, 0.0, 0.0, 0.5, -0.5, 0.0, 0.0)
    #     self.assertEqual(result, expected_result)

    # def test_inverse(self):
    #     matrix = M44f(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16)
    #     result = matrix.inverse()
    #     expected_result = M44f(-2.0, 1.0, 0.0, 0.0, 1.5, -1.0, 0.5, 0.0, -1.0, 1.0, 0.0, 0.0, 0.5, -0.5, 0.0, 0.0)
    #     self.assertEqual(result, expected_result)

    # def test_invert(self):
    #     matrix = M44f(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16)
    #     result = matrix.invert()
    #     expected_result = M44f(-2.0, 1.0, 0.0, 0.0, 1.5, -1.0, 0.5, 0.0, -1.0, 1.0, 0.0, 0.0, 0.5, -0.5, 0.0, 0.0)
    #     self.assertEqual(result, expected_result)

    # def test_makeIdentity(self):
    #     result = M44f.makeIdentity()
    #     expected_result = M44f(1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1)
    #     self.assertEqual(result, expected_result)

    # def test_minorOf(self):
    #     matrix = M44f(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16)
    #     result = M44f.minorOf(matrix, 1, 2, 3, 1)

    #     expected_result = 0.0
    #     self.assertEqual(result, expected_result)

    def test_multDirMatrix(self):
        direction = V3f(1, 0, 0)
        matrix = M44f().rotate(Eulerf(45, 0, 0))
        result = matrix.multDirMatrix(direction)
        expected_result = V3f(1, 0, 0)
        self.assertEqual(result, expected_result)

    def test_multVecMatrix(self):
        vector = V3f(1, 2, 3)
        matrix = M44f().setScale(V3f(2, 3, 4))
        result = matrix.multVecMatrix(vector)
        expected_result = V3f(2, 6, 12)
        self.assertEqual(result, expected_result)

    def test_negate(self):
        matrix = M44f(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16)
        result = M44f.negate(matrix)
        expected_result = M44f(-1, -2, -3, -4, -5, -6, -7, -8, -9, -10, -11, -12, -13, -14, -15, -16)
        self.assertEqual(result, expected_result)

    def test_removeScaling(self):
        matrix = M44f().setScale(V3f(2, 3, 4))
        matrix.removeScaling()
        self.assertEqual(matrix, M44f())

    # def test_removeScalingAndShear(self):
    #     matrix = M44f().setScale(V3f(2, 3, 4)).setShear(V3f(0.1, 0.2, 0.3)).setTranslation(V3f(1, 2, 3))
    #     result = M44f.removeScalingAndShear(matrix)

    #     expected_result = M44f().setTranslation(V3f(1, 2, 3))
    #     self.assertEqual(result, expected_result)

    # def test_rotate(self):
    #     matrix = M44f().rotate(Eulerf(45, 0, 0))
    #     vector = V3f(1, 0, 0)
    #     result = matrix.rotate(vector)
    #     expected_result = V3f(1, 0, 0)
    #     self.assertEqual(result, expected_result)

    # def test_rotationMatrix(self):
    #     angles = Eulerf(45, 30, 15)
    #     result = M44f.rotationMatrix(angles)
    #     expected_result = M44f().rotate(angles)
    #     self.assertEqual(result, expected_result)

    # def test_rotationMatrixWithUpDir(self):
    #     angles = Eulerf(45, 30, 15)
    #     up_direction = V3f(0, 1, 0)
    #     result = M44f.rotationMatrixWithUpDir(angles, up_direction)
    #     expected_result = M44f().rotate(angles)
    #     self.assertEqual(result, expected_result)

    def test_sansScaling(self):
        matrix = M44f().setScale(V3f(2, 3, 4)).setTranslation(V3f(1, 2, 3))
        result = M44f.sansScaling(matrix)
        expected_result = M44f().setTranslation(V3f(1, 2, 3))
        self.assertEqual(result, expected_result)

    # def test_sansScalingAndShear(self):
    #     matrix = M44f().setScale(V3f(2, 3, 4)).setShear(V3f(0.1, 0.2, 0.3)).setTranslation(V3f(1, 2, 3))
    #     result = M44f.sansScalingAndShear(matrix)
    #     expected_result = M44f().setTranslation(V3f(1, 2, 3))
    #     self.assertEqual(result, expected_result)

    # def test_scale(self):
    #     matrix = M44f().setScale(V3f(2, 3, 4))
    #     vector = V3f(1, 1, 1)
    #     result = matrix.scale(vector)
    #     expected_result = V3f(2, 3, 4)
    #     self.assertEqual(result, expected_result)

    # def test_setScale(self):
    #     scale_vector = V3f(2, 3, 4)
    #     result = M44f().setScale(scale_vector)
    #     expected_result = M44f(2, 0, 0, 0, 0, 3, 0, 0, 0, 0, 4, 0, 0, 0, 0, 1)
    #     self.assertEqual(result, expected_result)

    # def test_setShear(self):
    #     shear_vector = V3f(1, 2, 3)
    #     result = M44f().setShear(shear_vector)
    #     expected_result = M44f(1, 0, 0, 0, 2, 0, 0, 3, 0, 0, 1, 0, 0, 0, 0, 1)
    #     self.assertEqual(result, expected_result)

    # def test_setTranslation(self):
    #     translation_vector = V3f(1, 2, 3)
    #     result = M44f().setTranslation(translation_vector)
    #     expected_result = M44f(1, 0, 0, 1, 0, 1, 0, 2, 0, 0, 1, 3, 0, 0, 0, 1)
    #     self.assertEqual(result, expected_result)

    def test_setValue(self):
        values = M44f(
            1, 2, 3, 4,
            5, 6, 7, 8,
            9, 10, 11, 12,
            13, 14, 15, 16
        )
        result = M44f()
        result.setValue(values)
        expected_result = M44f(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16)
        self.assertEqual(result, expected_result)

    # def test_shear(self):
    #     shear_vector = V3f(1, 2, 3)
    #     result = M44f()
    #     result.setShear(shear_vector)
    #     expected_result = M44f(1, 0, 0, 0, 2, 0, 0, 3, 0, 0, 1, 0, 0, 0, 0, 1)
    #     self.assertEqual(result, expected_result)

    def test_singularValueDecomposition(self):
        """
        TODO this
        """
        matrix = M44f(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16)
        u, sigma, v_t = M44f.singularValueDecomposition(matrix, True)
        # SVD implementation uses two-sided Jacobi rotations to iteratively diagonalize the matrix
        # M44f, V4f, M44f
        # expected_u = M44f()
        # expected_sigma = M44f()
        # expected_v_t = M44f()
        # self.assertEqual(u, expected_u)
        # self.assertEqual(sigma, expected_sigma)
        # self.assertEqual(v_t, expected_v_t)

    # def test_symmetricEigensolve(self):
    #     matrix = M44f(1, 2, 3, 4, 2, 5, 6, 7, 3, 6, 8, 9, 4, 7, 9, 10)
    #     eigenvectors, eigenvalues = M44f.symmetricEigensolve(matrix)
    #     expected_eigenvectors = [V3f(), V3f(), V3f(), V3f()]
    #     expected_eigenvalues = V3f()
    #     self.assertEqual(eigenvectors, expected_eigenvectors)
    #     self.assertEqual(eigenvalues, expected_eigenvalues)

    def test_translate(self):
        matrix = M44f().setTranslation(V3f(1, 2, 3))
        vector = V3f(2, 3, 4)
        result = matrix.translate(vector)
        expected_result = V3f(3, 5, 7)
        self.assertEqual(result.translation(), expected_result)

    def test_translation(self):
        matrix = M44f().setTranslation(V3f(1, 2, 3))
        result = matrix.translation()
        expected_result = V3f(1, 2, 3)
        self.assertEqual(result, expected_result)

    def test_transpose(self):
        matrix = M44f(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16)
        result = M44f.transpose(matrix)
        expected_result = M44f(1, 5, 9, 13, 2, 6, 10, 14, 3, 7, 11, 15, 4, 8, 12, 16)
        self.assertEqual(result, expected_result)

    def test_transposed(self):
        matrix = M44f(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16)
        result = matrix.transposed()
        expected_result = M44f(1, 5, 9, 13, 2, 6, 10, 14, 3, 7, 11, 15, 4, 8, 12, 16)
        self.assertEqual(result, expected_result)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

import os, sys, site, glob
from pathlib import Path

def print_files_in_path(var):
    path_env = os.environ.get(var, '')
    print(f"{var}={path_env}")
    for directory in path_env.split(os.pathsep):
        if directory:
            directory = os.path.normpath(directory)
            if os.path.isdir(directory):
                for file in os.listdir(directory):
                    full_path = os.path.join(directory, file)
                    if os.path.isfile(full_path):
                        print(f'    {full_path}')

def test_import():

    try:
        import pybindimath
        print()
        print(f"pybindimath imported from {pybindimath.__file__}")
        print("ok")
    except Exception as e:
        print(f"{__file__} failed. Can't import pybindimath: {e}")
        #print_files_in_path('PATH')
        #print_files_in_path('PYTHONPATH')
        sys.exit(1)
        
test_import()



#!/usr/bin/env python3
# SPDX-License-Identifier: BSD-3-Clause
# Copyright Contributors to the OpenColorIO Project.

#
# This script compares an install_manifest.txt from a CI build and
# compares it against a known valid manifest to verify that the proper
# files have been installed. Both missing files and extra files not
# expected to be installed are reported as errors.
#
# This script is invoked by the .github/workflows/ci_steps.yml workflow.
#
# usage: validate_install.py <generated manifest> <archived manifest> _build/CMakeCache.txt
#
# Ths script must be run from the source directory.
#
# The first argument is the candidate manifest from the build in
# question.  The second argument is the reference manifest from
# share/ci/install_manifest, known to be correct. The third argument
# is the CMakeCache.txt file from the build directory.
#
# The reference manifest may contain variables for the library version:
#   $MAJOR = IMATH_VERSION_MAJOR
#   $MINOR = IMATH_VERSION_MINOR
#   $PATCH = IMATH_VERSION_PATCH
#   $SOVERSION = IMATH_LIB_SOVERSION
# The values of these variables are extracted from the CMakeLists.txt
# in the current diretory and the CMakeCache.txt file from the build 
# directory, provided as an argument. This is so the reference manifests
# don't have be updated on every release, sinced these values appear
# in the installed library names.
#
# The manifests *must* be updated whenever files are added to what is
# installed: new headers, libraries, tools, cmake, etc, or a change to
# file names. Note that this validates just the file *names*, not the
# contents.
#
# The CI uploads the install manifests as job artifacts, attached to
# the GitHub status page.
#
# NOTE: If you add a new installed header, you must add it to each of
# the reference manifest in share/ci/install_manifest.
#
# The first line of the manifest file is the cmake command for the
# build that generated it, just to help identify the contents.
#

import os
import sys
import re
import argparse
from pathlib import Path

def imath_soversion():

    with open("CMakeLists.txt", "r") as f:
        for line in f:
            m = re.search(r"set\(IMATH_LIB_SOVERSION (\d+)\)", line)
            if m:
                return m.group(1)

    print("Can't find soversion")
    sys.exit(1)

def imath_version(CMakeCache):
    major = minor = patch = None
    with open(CMakeCache, 'r') as f:
        for line in f:
            if line.startswith('CMAKE_PROJECT_VERSION_MAJOR:STATIC='):
                major = line.split('=')[1].strip()
            elif line.startswith('CMAKE_PROJECT_VERSION_MINOR:STATIC='):
                minor = line.split('=')[1].strip()
            elif line.startswith('CMAKE_PROJECT_VERSION_PATCH:STATIC='):
                patch = line.split('=')[1].strip()
            if major != None and minor != None and patch != None:
                return major, minor, patch
                
    print("Can't find Imath version!")
    sys.exit(1)

def process_line(line, major, minor, patch, so):
    path = line.strip().split("/_install/", 1)[-1]
    path = path.replace("lib64", "lib")  # ignore differences between "lib64" and "lib"
    path = path.replace("$MAJOR", major)
    path = path.replace("$MINOR", minor)
    path = path.replace("$PATCH", patch)
    path = path.replace("$SOVERSION", so)
    path = path.replace("$PYTHONMAJOR", str(sys.version_info.major))
    path = path.replace("$PYTHONMINOR", str(sys.version_info.minor))
    return path

def load_manifest(path, major, minor, patch, so):
    """Load and return the list of files from the install manifest."""
    with open(path, 'r') as file:
        return sorted(process_line(line, major, minor, patch, so) for line in file if line.strip() and not line.lstrip().startswith('#'))

def validate_install(build_manifest_path, reference_manifest_path, CMakeCache):
    """Main function to verify the installed files."""

    major, minor, patch = imath_version(CMakeCache)
    so = imath_soversion()

    build_manifest = load_manifest(build_manifest_path, major, minor, patch, so)
    reference_manifest = load_manifest(reference_manifest_path, major, minor, patch, so)

    print(f"imath version: {major}.{minor}.{patch} soversion={so}")
    print(f"python version: {sys.version_info.major}.{sys.version_info.minor}")
    print("reference_manifest:")
    for l in reference_manifest:
        print(f"  {l}")
    print("build_manifest:")
    for l in build_manifest:
        print(f"  {l}")

    # Compare manifests
    missing_files = sorted(set(reference_manifest) - set(build_manifest))
    extra_files = sorted(set(build_manifest) - set(reference_manifest))

    # Output results
    if missing_files:
        print("Error: The following files should have been installed but weren't:\n  " + '\n  '.join(missing_files))
    if extra_files:
        print("Error: The following files were installed but were not expected:\n  " + '\n  '.join(extra_files))

    if missing_files or extra_files:
        return 1

    print("install manifest is valid.")

    return 0

if __name__ == "__main__":

    print(f"validate_install: {sys.argv}")

    parser = argparse.ArgumentParser(description="Validate installed files against reference install manifest.")
    parser.add_argument("build_manifest", help="Path to the build install_manifest.txt")
    parser.add_argument("reference_manifest", help="Path to the reference install_manifest.txt")
    parser.add_argument("CMakeCache", help="CakeCache.txt file path")
    args = parser.parse_args()

    if not os.path.exists(args.build_manifest):
        print(f"build manifest does not exist: {args.build_manifest}")
        sys.exit(1)

    if not os.path.exists(args.reference_manifest):
        print(f"reference manifest does not exist: {args.reference_manifest}")
        sys.exit(1)

    print(f"build_manifest={args.build_manifest}")
    print(f"reference_manifest={args.reference_manifest}")

    status = validate_install(args.build_manifest, args.reference_manifest, args.CMakeCache)

    sys.exit(status)




# SPDX-License-Identifier: BSD-3-Clause
# Copyright Contributors to the OpenEXR Project.

import unittest
from imath import V3f

class V3fTestCase(unittest.TestCase):
    def test_baseTypeEpsilon(self):
        result = V3f.baseTypeEpsilon()
        self.assertEqual(result, 1.1920928955078125e-07)

    def test_baseTypeLowest(self):
        result = V3f.baseTypeLowest()
        self.assertEqual(result, -3.4028234663852886e+38)

    def test_baseTypeMax(self):
        result = V3f.baseTypeMax()
        self.assertEqual(result, 3.4028234663852886e+38)

    def test_baseTypeSmallest(self):
        result = V3f.baseTypeSmallest()
        self.assertEqual(result, 1.1754943508222875e-38)

    def test_closestVertex(self):
        vector = V3f(1, 2, 3)
        p1, p2, p3 =  V3f(1, 0, 0), V3f(0, 1, 0), V3f(0, 0, 1)
        result = V3f.closestVertex(vector, p1, p2, p3)
        expected_result = V3f(0, 0, 1)
        self.assertEqual(result, expected_result)

    def test_cross(self):
        vector1 = V3f(1, 2, 3)
        vector2 = V3f(4, 5, 6)
        result = V3f.cross(vector1, vector2)
        expected_result = V3f(-3, 6, -3)
        self.assertEqual(result, expected_result)

    def test_dimensions(self):
        result = V3f.dimensions()
        self.assertEqual(result, 3)

    def test_dot(self):
        vector1 = V3f(1, 2, 3)
        vector2 = V3f(4, 5, 6)
        result = V3f.dot(vector1, vector2)
        expected_result = 32
        self.assertEqual(result, expected_result)

    def test_equalWithAbsError(self):
        vector1 = V3f(1, 2, 3)
        vector2 = V3f(1.01, 2.02, 3.01)
        result = V3f.equalWithAbsError(vector1, vector2, 0.05)
        self.assertTrue(result)

    def test_equalWithRelError(self):
        vector1 = V3f(1, 2, 3)
        vector2 = V3f(1.01, 2.02, 3.01)
        result = V3f.equalWithRelError(vector1, vector2, 0.02)
        self.assertTrue(result)

    def test_length(self):
        vector = V3f(2, 4, 4)
        result = vector.length()
        expected_result = 6
        self.assertAlmostEqual(result, expected_result)

    def test_length2(self):
        vector = V3f(1, 2, 3)
        result = vector.length2()
        expected_result = 14
        self.assertEqual(result, expected_result)

    def test_negate(self):
        vector = V3f(1, 2, 3)
        vector.negate()
        expected_result = V3f(-1, -2, -3)
        self.assertEqual(vector, expected_result)

    def test_normalize(self):
        vector = V3f(1, 2, 3)
        normal_result = vector.normalize()
        expected_result = V3f(0.267261, 0.534522, 0.801783)
        result = V3f.equalWithAbsError(normal_result, expected_result, 0.01)
        self.assertTrue(result)

    def test_normalizeExc(self):
        vector = V3f(1, 2, 3)
        normal_result = vector.normalizeExc()
        expected_result = V3f(0.267261, 0.534522, 0.801783)
        result = V3f.equalWithAbsError(normal_result, expected_result, 0.01)
        self.assertTrue(result)

    def test_normalizeNonNull(self):
        vector = V3f(1, 2, 3)
        normal_result = vector.normalizeNonNull()
        expected_result = V3f(0.267261, 0.534522, 0.801783)
        result = V3f.equalWithAbsError(normal_result, expected_result, 0.01)
        self.assertTrue(result)

    def test_normalized(self):
        vector = V3f(1, 2, 3)
        normal_result = vector.normalized()
        expected_result = V3f(0.267261, 0.534522, 0.801783)
        result = V3f.equalWithAbsError(normal_result, expected_result, 0.01)
        self.assertTrue(result)

    def test_normalizedExc(self):
        vector = V3f(1, 2, 3)
        normal_result = vector.normalizedExc()
        expected_result = V3f(0.267261, 0.534522, 0.801783)
        result = V3f.equalWithAbsError(normal_result, expected_result, 0.01)
        self.assertTrue(result)

    def test_normalizedNonNull(self):
        vector = V3f(1, 2, 3)
        normal_result = vector.normalizedNonNull()
        expected_result = V3f(0.267261, 0.534522, 0.801783)
        result = V3f.equalWithAbsError(normal_result, expected_result, 0.01)
        self.assertTrue(result)

    def test_orthogonal(self):
        vector1 = V3f(2, 2, 0)
        vector2 = V3f(0, 1, 0)
        orthogonal_vec = V3f.orthogonal(vector1, vector2)
        expected_result = V3f(-0.5, 0.5, 0)
        result = V3f.equalWithAbsError(orthogonal_vec, expected_result, 0.01)
        self.assertTrue(result)

    def test_project(self):
        vector = V3f(1, 2, 3)
        direction = V3f(1, 0, 0)
        result = V3f.project(vector, direction)
        expected_result = V3f(1, 0, 0)
        self.assertEqual(result, expected_result)

    def test_reflect(self):
        vector = V3f(4, 5, 6)
        normal = V3f(1, 0, 0)
        result = V3f.reflect(vector, normal)
        expected_result = V3f(4, -5, -6)
        self.assertEqual(result, expected_result)

    def test_xyz(self):
        vector = V3f(1, 2, 3)
        self.assertEqual(vector.x, 1)
        self.assertEqual(vector.y, 2)
        self.assertEqual(vector.z, 3)

if __name__ == '__main__':
    unittest.main()
